//
//  StatusesTableViewController.swift
//  Tusker
//
//  Created by Shadowfacts on 8/15/18.
//  Copyright © 2018 Shadowfacts. All rights reserved.
//

import UIKit
import MastodonKit
import SwiftSoup

class StatusesTableViewController: UITableViewController {

    var statuses: [Status] = [] {
        didSet {
            DispatchQueue.main.async {
                self.tableView.reloadData()
            }
        }
    }
    
    var newer: RequestRange?
    var older: RequestRange?
    
    override func viewWillAppear(_ animated: Bool) {
        guard MastodonController.shared.client?.accessToken != nil else { return }
        MastodonController.shared.client.run(Timelines.home()) { result in
            guard case let .success(statuses, pagination) = result else { fatalError() }
            self.statuses = statuses
            self.newer = pagination?.previous
            self.older = pagination?.next
            DispatchQueue.main.async {
                self.tableView.reloadData()
            }
        }
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        tableView.rowHeight = UITableView.automaticDimension
        tableView.estimatedRowHeight = 140
    }

    /*
     // MARK: - Navigation
     
     // In a storyboard-based application, you will often want to do a little preparation before navigation
     override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
     // Get the new view controller using segue.destination.
     // Pass the selected object to the new view controller.
     }
     */
    
    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return statuses.count
    }

    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let cell = tableView.dequeueReusableCell(withIdentifier: "statusCell", for: indexPath) as? StatusTableViewCell else {
            fatalError()
        }
        
        let status = statuses[indexPath.row]
        
        cell.updateUI(for: status)

        return cell
    }
    
    override func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        if indexPath.row == statuses.count - 1 {
            guard let older = older else { return }
            
            MastodonController.shared.client.run(Timelines.home(range: older)) { result in
                guard case let .success(newStatuses, pagination) = result else { fatalError() }
                self.older = pagination?.next
                self.statuses.append(contentsOf: newStatuses)
            }
        }
    }

    @IBAction func refreshStatuses(_ sender: Any) {
        guard let newer = newer else { return }
        
        MastodonController.shared.client.run(Timelines.home(range: newer)) { result in
            guard case let .success(newStatuses, pagination) = result else { fatalError() }
            self.newer = pagination?.previous
            self.statuses.insert(contentsOf: newStatuses, at: 0)
            DispatchQueue.main.async {
                self.refreshControl?.endRefreshing()
                
                // maintain the current position in the list (don't scroll to the top)
                self.tableView.scrollToRow(at: IndexPath(row: newStatuses.count, section: 0), at: .top, animated: false)
            }
        }
    }
    
}
